@@ -2,9 +2,13 @@ package Module::Starter::Simple;
 # vi:et:sw=4 ts=4
 
 use strict;
+use warnings;
 
 use ExtUtils::Command qw( rm_rf mkpath touch );
-use File::Spec;
+use File::Spec ();
+use Carp qw( carp confess croak );
+
+use Module::Starter::BuilderSet;
 
 =head1 NAME
 
@@ -12,15 +16,15 @@ Module::Starter::Simple - a simple, comprehensive Module::Starter plugin
 
 =head1 VERSION
 
-Version 1.42
+Version 1.44
 
 =cut
 
-our $VERSION = '1.42';
+our $VERSION = '1.44';
 
 =head1 SYNOPSIS
 
- use Module::Starter qw(Module::Starter::Simple);
+  use Module::Starter qw(Module::Starter::Simple);
 
  Module::Starter->create_distro(%args);
 
@@ -45,14 +49,14 @@ sub create_distro {
     my $self = $class->new( @_ );
 
     my $modules = $self->{modules} || [];
-    my @modules = map { split /,/ } @$modules;
-    die "No modules specified.\n" unless @modules;
+    my @modules = map { split /,/ } @{$modules};
+    croak "No modules specified.\n" unless @modules;
     for (@modules) {
-        die "Invalid module name: $_" unless /\A[a-z_]\w*(?:::[\w]+)*\Z/i;
+        croak "Invalid module name: $_" unless /\A[a-z_]\w*(?:::[\w]+)*\Z/i;
     }
 
-    die "Must specify an author\n" unless $self->{author};
-    die "Must specify an email address\n" unless $self->{email};
+    croak "Must specify an author\n" unless $self->{author};
+    croak "Must specify an email address\n" unless $self->{email};
     ($self->{email_obfuscated} = $self->{email}) =~ s/@/ at /;
 
     $self->{license} ||= 'perl';
@@ -71,53 +75,16 @@ sub create_distro {
 
     push @files, $self->create_t( @modules );
     push @files, $self->create_cvsignore;
-
-    my @builders;
-    if (ref $self->{builder} eq "ARRAY") {
-        my %unique = map { $_ => 1 } @{$self->{builder}};
-        @builders = keys %unique;
-    }
-    else {
-        @builders = ($self->{builder});
-    }
-
-    # this block should be pulled out to its own sub
-    my @build_instructions;
-    for my $builder ( @builders ) {
-        next unless $builder;
-        if ( !@build_instructions ) {
-            push @build_instructions, "To install this module, run the following commands:";
-        }
-        else {
-            push @build_instructions, "Alternatively, to install with $builder, you can use the following commands:";
-        }
-        if ( $builder eq 'Module::Build' ) {
-            push @files, $self->create_Build_PL( $self->{main_module} );
-            push @build_instructions, <<'HERE';
-    perl Build.PL
-    ./Build
-    ./Build test
-    ./Build install
-HERE
-        }
-        else {
-            push @files, $self->create_Makefile_PL( $self->{main_module} );
-            push @build_instructions, <<'HERE';
-    perl Makefile.PL
-    make
-    make test
-    make install
-HERE
-        }
-    }
-
-    my $build_instructions = join( "\n\n", @build_instructions );
+    my %build_results = $self->create_build();
+    push(@files, @{ $build_results{files} } );
 
     push @files, $self->create_Changes;
-    push @files, $self->create_README( $build_instructions );
-    push @files, "MANIFEST";
+    push @files, $self->create_README( $build_results{instructions} );
+    push @files, 'MANIFEST';
     push @files, 'META.yml # Will be created by "make dist"';
     $self->create_MANIFEST( @files );
+
+    return;
 }
 
 =head2 C<< new(%args) >>
@@ -129,7 +96,10 @@ the attention of subclass authors.
 
 =cut
 
-sub new { my $class = shift; bless { @_ } => $class; }
+sub new {
+  my $class = shift;
+  return bless { @_ } => $class;
+}
 
 =head1 OBJECT METHODS
 
@@ -151,12 +121,15 @@ sub create_basedir {
 
     # Make sure there's no directory
     if ( -e $self->{basedir} ) {
-        die "$self->{basedir} already exists.  Use --force if you want to stomp on it.\n" unless $self->{force};
+        die( "$self->{basedir} already exists.  ".
+             "Use --force if you want to stomp on it.\n"
+            ) unless $self->{force};
 
         local @ARGV = $self->{basedir};
         rm_rf();
 
-        die "Couldn't delete existing $self->{basedir}: $!\n" if -e $self->{basedir};
+        die "Couldn't delete existing $self->{basedir}: $!\n"
+          if -e $self->{basedir};
     }
 
     CREATE_IT: {
@@ -167,6 +140,8 @@ sub create_basedir {
 
         die "Couldn't create $self->{basedir}: $!\n" unless -d $self->{basedir};
     }
+
+    return;
 }
 
 =head2 create_modules( @modules )
@@ -201,134 +176,22 @@ method eventually.)
 =cut
 
 sub _license_blurb {
-  my $self = shift;
-  my $license_blurb;
+    my $self = shift;
+    my $license_blurb;
 
-  if ($self->{license} eq 'perl') {
-    $license_blurb = <<EOT;
+    if ($self->{license} eq 'perl') {
+        $license_blurb = <<'EOT';
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 EOT
-  } else {
-    $license_blurb = <<EOT;
+    }
+    else {
+        $license_blurb = <<"EOT";
 This program is released under the following license: $self->{license}
 EOT
-  }
-  chomp $license_blurb;
-  return $license_blurb;
-}
-
-sub module_guts {
-    my $self = shift;
-    my $module = shift;
-    my $rtname = shift;
-
-    my $year = $self->_thisyear();
-    my $license_blurb = $self->_license_blurb();
-
-    my $content = <<"HERE";
-package $module;
-
-use warnings;
-use strict;
-
-\=head1 NAME
-
-$module - The great new $module!
-
-\=head1 VERSION
-
-Version 0.01
-
-\=cut
-
-our \$VERSION = '0.01';
-
-\=head1 SYNOPSIS
-
-Quick summary of what the module does.
-
-Perhaps a little code snippet.
-
-    use $module;
-
-    my \$foo = $module->new();
-    ...
-
-\=head1 EXPORT
-
-A list of functions that can be exported.  You can delete this section
-if you don't export anything, such as for a purely object-oriented module.
-
-\=head1 FUNCTIONS
-
-\=head2 function1
-
-\=cut
-
-sub function1 {
-}
-
-\=head2 function2
-
-\=cut
-
-sub function2 {
-}
-
-\=head1 AUTHOR
-
-$self->{author}, C<< <$self->{email_obfuscated}> >>
-
-\=head1 BUGS
-
-Please report any bugs or feature requests to
-C<bug-$rtname at rt.cpan.org>, or through the web interface at
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=$self->{distro}>.
-I will be notified, and then you'll automatically be notified of progress on
-your bug as I make changes.
-
-\=head1 SUPPORT
-
-You can find documentation for this module with the perldoc command.
-
-    perldoc $self->{main_module}
-
-You can also look for information at:
-
-\=over 4
-
-\=item * AnnoCPAN: Annotated CPAN documentation
-
-L<http://annocpan.org/dist/$self->{distro}>
-
-\=item * CPAN Ratings
-
-L<http://cpanratings.perl.org/d/$self->{distro}>
-
-\=item * RT: CPAN's request tracker
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=$self->{distro}>
-
-\=item * Search CPAN
-
-L<http://search.cpan.org/dist/$self->{distro}>
-
-\=back
-
-\=head1 ACKNOWLEDGEMENTS
-
-\=head1 COPYRIGHT & LICENSE
-
-Copyright $year $self->{author}, all rights reserved.
-
-$license_blurb
-
-\=cut
-
-1; # End of $module
-HERE
-    return $content;
+    }
+    chomp $license_blurb;
+    return $license_blurb;
 }
 
 # _create_module: used by create_modules to build each file and put data in it
@@ -339,9 +202,10 @@ sub _create_module {
     my $rtname = shift;
 
     my @parts = split( /::/, $module );
-    my $filepart = (pop @parts) . ".pm";
+    my $filepart = (pop @parts) . '.pm';
     my @dirparts = ( $self->{basedir}, 'lib', @parts );
-    my $manifest_file = join( "/", "lib", @parts, $filepart );
+    my $SLASH = q{/};
+    my $manifest_file = join( $SLASH, 'lib', @parts, $filepart );
     if ( @dirparts ) {
         my $dir = File::Spec->catdir( @dirparts );
         if ( not -d $dir ) {
@@ -353,19 +217,47 @@ sub _create_module {
 
     my $module_file = File::Spec->catfile( @dirparts,  $filepart );
 
-    $self->{module_file}{$module} =
-        File::Spec->catfile('lib', @parts, $filepart);
-
-    open( my $fh, ">", $module_file ) or die "Can't create $module_file: $!\n";
-    print $fh $self->module_guts( $module, $rtname );
-    close $fh;
+    $self->{module_file}{$module} = File::Spec->catfile('lib', @parts, $filepart);
+    $self->create_file( $module_file, $self->module_guts( $module, $rtname ) );
     $self->progress( "Created $module_file" );
 
     return $manifest_file;
 }
 
-sub _thisyear { (localtime())[5] + 1900 }
+sub _thisyear {
+    return (localtime())[5] + 1900;
+}
 
+sub _module_to_pm_file {
+    my $self = shift;
+    my $module = shift;
+
+    my @parts = split( /::/, $module );
+    my $pm = pop @parts;
+    my $pm_file = File::Spec->catfile( 'lib', @parts, "${pm}.pm" );
+    $pm_file =~ s{\\}{/}g; # even on Win32, use forward slash
+
+    return $pm_file;
+}
+
+sub _reference_links {
+  return
+    ( { nickname => 'RT',
+        title    => 'CPAN\'s request tracker',
+        link     => 'http://rt.cpan.org/NoAuth/Bugs.html?Dist=%s',
+      },
+      { nickname => 'AnnoCPAN',
+        title    => 'Annotated CPAN documentation',
+        link     => 'http://annocpan.org/dist/%s',
+      },
+      { title    => 'CPAN Ratings',
+        link     => 'http://cpanratings.perl.org/d/%s',
+      },
+      { title    => 'Search CPAN',
+        link     => 'http://search.cpan.org/dist/%s',
+      },
+    );
+}
 
 =head2 create_Makefile_PL( $main_module )
 
@@ -375,23 +267,38 @@ named in I<$main_module> as the main module of the distribution.
 =cut
 
 sub create_Makefile_PL {
-    my $self = shift;
-    my $main_module = shift;
+    my $self         = shift;
+    my $main_module  = shift;
+    my $builder_name = 'ExtUtils::MakeMaker';
+    my $output_file  =
+      Module::Starter::BuilderSet->new()->file_for_builder($builder_name);
+    my $fname        = File::Spec->catfile( $self->{basedir}, $output_file );
 
-    my @parts = split( /::/, $main_module );
-    my $pm = pop @parts;
-    my $main_pm_file = File::Spec->catfile( "lib", @parts, "${pm}.pm" );
-       $main_pm_file =~ s{\\}{/}g; # even on Win32, use forward slash
-
-    my $fname = File::Spec->catfile( $self->{basedir}, "Makefile.PL" );
-    open( my $fh, ">", $fname ) or die "Can't create $fname: $!\n";
-
-    print $fh $self->Makefile_PL_guts($main_module, $main_pm_file);
-
-    close $fh;
+    $self->create_file( $fname, $self->Makefile_PL_guts($main_module) );
     $self->progress( "Created $fname" );
 
-    return "Makefile.PL";
+    return $output_file;
+}
+
+=head2 create_MI_Makefile_PL( $main_module )
+
+This will create a Module::Install Makefile.PL for the distribution, and will
+use the module named in I<$main_module> as the main module of the distribution.
+
+=cut
+
+sub create_MI_Makefile_PL {
+    my $self         = shift;
+    my $main_module  = shift;
+    my $builder_name = 'Module::Install';
+    my $output_file  =
+      Module::Starter::BuilderSet->new()->file_for_builder($builder_name);
+    my $fname        = File::Spec->catfile( $self->{basedir}, $output_file );
+
+    $self->create_file( $fname, $self->MI_Makefile_PL_guts($main_module) );
+    $self->progress( "Created $fname" );
+
+    return $output_file;
 }
 
 =head2 Makefile_PL_guts( $main_module, $main_pm_file )
@@ -405,7 +312,8 @@ module, I<$main_module>.
 sub Makefile_PL_guts {
     my $self = shift;
     my $main_module = shift;
-    my $main_pm_file = shift;
+
+    my $main_pm_file = $self->_module_to_pm_file($main_module);
 
     (my $author = "$self->{author} <$self->{email}>") =~ s/'/\'/g;
 
@@ -430,6 +338,40 @@ HERE
 
 }
 
+=head2 MI_Makefile_PL_guts( $main_module, $main_pm_file )
+
+This method is called by create_MI_Makefile_PL and returns text used to populate
+Makefile.PL; I<$main_pm_file> is the filename of the distribution's main
+module, I<$main_module>.
+
+=cut
+
+sub MI_Makefile_PL_guts {
+    my $self = shift;
+    my $main_module = shift;
+    my $module_name = $main_module;
+    $module_name =~ s/::/-/g;
+
+    my $main_pm_file = $self->_module_to_pm_file($main_module);
+
+    (my $author = "$self->{author} <$self->{email}>") =~ s/'/\'/g;
+
+    return <<"HERE";
+use inc::Module::Install;
+
+name '$module_name';
+all_from '$main_pm_file';
+
+build_requires 'Test::More';
+
+auto_install;
+
+WriteAll;
+
+HERE
+
+}
+
 =head2 create_Build_PL( $main_module )
 
 This will create the Build.PL for the distribution, and will use the module
@@ -438,23 +380,17 @@ named in I<$main_module> as the main module of the distribution.
 =cut
 
 sub create_Build_PL {
-    my $self = shift;
-    my $main_module = shift;
+    my $self         = shift;
+    my $main_module  = shift;
+    my $builder_name = 'Module::Build';
+    my $output_file  =
+      Module::Starter::BuilderSet->new()->file_for_builder($builder_name);
+    my $fname        = File::Spec->catfile( $self->{basedir}, $output_file );
 
-    my @parts = split( /::/, $main_module );
-    my $pm = pop @parts;
-    my $main_pm_file = File::Spec->catfile( "lib", @parts, "${pm}.pm" );
-       $main_pm_file =~ s{\\}{/}g; # even on Win32, use forward slash
-
-    my $fname = File::Spec->catfile( $self->{basedir}, "Build.PL" );
-    open( my $fh, ">", $fname ) or die "Can't create $fname: $!\n";
-
-    print $fh $self->Build_PL_guts($main_module, $main_pm_file);
-
-    close $fh;
+    $self->create_file( $fname, $self->Build_PL_guts($main_module) );
     $self->progress( "Created $fname" );
 
-    return "Build.PL";
+    return $output_file;
 }
 
 =head2 Build_PL_guts( $main_module, $main_pm_file )
@@ -468,7 +404,8 @@ I<$main_module>.
 sub Build_PL_guts {
     my $self = shift;
     my $main_module = shift;
-    my $main_pm_file = shift;
+
+    my $main_pm_file = $self->_module_to_pm_file($main_module);
 
     (my $author = "$self->{author} <$self->{email}>") =~ s/'/\'/g;
 
@@ -486,6 +423,7 @@ my \$builder = Module::Build->new(
         'Test::More' => 0,
     },
     add_to_cleanup      => [ '$self->{distro}-*' ],
+    create_makefile_pl => 'traditional',
 );
 
 \$builder->create_build_script();
@@ -502,15 +440,11 @@ This method creates a skeletal Changes file.
 sub create_Changes {
     my $self = shift;
 
-    my $fname = File::Spec->catfile( $self->{basedir}, "Changes" );
-    open( my $fh, ">", $fname ) or die "Can't create $fname: $!\n";
+    my $fname = File::Spec->catfile( $self->{basedir}, 'Changes' );
+    $self->create_file( $fname, $self->Changes_guts() );
+    $self->progress( "Created $fname" );
 
-    print $fh $self->Changes_guts();
-
-    close $fh;
-    $self->verbose( "Created $fname" );
-
-    return "Changes";
+    return 'Changes';
 }
 
 =head2 Changes_guts
@@ -541,15 +475,11 @@ sub create_README {
     my $self = shift;
     my $build_instructions = shift;
 
-    my $fname = File::Spec->catfile( $self->{basedir}, "README" );
-    open( my $fh, ">", $fname ) or die "Can't create $fname: $!\n";
+    my $fname = File::Spec->catfile( $self->{basedir}, 'README' );
+    $self->create_file( $fname, $self->README_guts($build_instructions) );
+    $self->progress( "Created $fname" );
 
-    print $fh $self->README_guts($build_instructions);
-
-    close $fh;
-    $self->verbose( "Created $fname" );
-
-    return "README";
+    return 'README';
 }
 
 =head2 README_guts
@@ -558,16 +488,10 @@ Called by create_README, this method returns content for the README file.
 
 =cut
 
-sub README_guts {
+sub _README_intro {
     my $self = shift;
-    my $build_instructions = shift;
-
-    my $year = $self->_thisyear();
-    my $license_blurb = $self->_license_blurb();
-
-return <<"HERE";
-$self->{distro}
 
+    return <<"HERE";
 The README is used to introduce the module and provide instructions on
 how to install the module, any machine dependencies it may have (for
 example C compilers and installed libraries) and any other information
@@ -575,9 +499,61 @@ that should be provided before the module is installed.
 
 A README file is required for CPAN modules since CPAN extracts the README
 file from a module distribution so that people browsing the archive
-can use it get an idea of the modules uses. It is usually a good idea
+can use it to get an idea of the module's uses. It is usually a good idea
 to provide version information here so that people can decide whether
 fixes for the module are worth downloading.
+HERE
+}
+
+sub _README_information {
+    my $self = shift;
+
+    my @reference_links = _reference_links();
+
+    my $content = "You can also look for information at:\n";
+
+    foreach my $ref (@reference_links){
+      my $title;
+      $title = "$ref->{nickname}, " if exists $ref->{nickname};
+      $title .= $ref->{title};
+      my $link  = sprintf($ref->{link}, $self->{distro});
+
+      $content .= qq[
+    $title
+        $link
+];
+    }
+
+    return $content;
+}
+
+sub _README_license {
+    my $self = shift;
+
+    my $year          = $self->_thisyear();
+    my $license_blurb = $self->_license_blurb();
+
+return <<"HERE";
+COPYRIGHT AND LICENCE
+
+Copyright (C) $year $self->{author}
+
+$license_blurb
+HERE
+}
+
+sub README_guts {
+    my $self = shift;
+    my $build_instructions = shift;
+
+    my $intro         = $self->_README_intro();
+    my $information   = $self->_README_information();
+    my $license       = $self->_README_license();
+
+return <<"HERE";
+$self->{distro}
+
+$intro
 
 INSTALLATION
 
@@ -585,29 +561,14 @@ $build_instructions
 
 SUPPORT AND DOCUMENTATION
 
-After installing, you can find documentation for this module with the perldoc command.
+After installing, you can find documentation for this module with the
+perldoc command.
 
     perldoc $self->{main_module}
 
-You can also look for information at:
+$information
 
-    Search CPAN
-        http://search.cpan.org/dist/$self->{distro}
-
-    CPAN Request Tracker:
-        http://rt.cpan.org/NoAuth/Bugs.html?Dist=$self->{distro}
-
-    AnnoCPAN, annotated CPAN documentation:
-        http://annocpan.org/dist/$self->{distro}
-
-    CPAN Ratings:
-        http://cpanratings.perl.org/d/$self->{distro}
-
-COPYRIGHT AND LICENCE
-
-Copyright (C) $year $self->{author}
-
-$license_blurb
+$license
 HERE
 }
 
@@ -648,18 +609,36 @@ sub t_guts {
     $t_files{'pod.t'} = <<'HERE';
 #!perl -T
 
+use strict;
+use warnings;
 use Test::More;
-eval "use Test::Pod 1.14";
-plan skip_all => "Test::Pod 1.14 required for testing POD" if $@;
+
+# Ensure a recent version of Test::Pod
+my $min_tp = 1.22;
+eval "use Test::Pod $min_tp";
+plan skip_all => "Test::Pod $min_tp required for testing POD" if $@;
+
 all_pod_files_ok();
 HERE
 
     $t_files{'pod-coverage.t'} = <<'HERE';
-#!perl -T
-
+use strict;
+use warnings;
 use Test::More;
-eval "use Test::Pod::Coverage 1.04";
-plan skip_all => "Test::Pod::Coverage 1.04 required for testing POD coverage" if $@;
+
+# Ensure a recent version of Test::Pod::Coverage
+my $min_tpc = 1.08;
+eval "use Test::Pod::Coverage $min_tpc";
+plan skip_all => "Test::Pod::Coverage $min_tpc required for testing POD coverage"
+    if $@;
+
+# Test::Pod::Coverage doesn't require a minimum Pod::Coverage version,
+# but older versions don't recognize some common documentation styles
+my $min_pc = 0.18;
+eval "use Pod::Coverage $min_pc";
+plan skip_all => "Pod::Coverage $min_pc required for testing POD coverage"
+    if $@;
+
 all_pod_coverage_ok();
 HERE
 
@@ -679,6 +658,10 @@ $use_lines
 diag( "Testing $main_module \$${main_module}::VERSION, Perl \$], \$^X" );
 HERE
 
+    my $module_boilerplate_tests;
+    $module_boilerplate_tests .=
+      "  module_boilerplate_ok('$self->{module_file}{$_}');\n" for @modules;
+
     my $boilerplate_tests = @modules + 2 + $[;
     $t_files{'boilerplate.t'} = <<"HERE";
 #!perl -T
@@ -689,7 +672,7 @@ use Test::More tests => $boilerplate_tests;
 
 sub not_in_file_ok {
     my (\$filename, \%regex) = \@_;
-    open my \$fh, "<", \$filename
+    open( my \$fh, '<', \$filename )
         or die "couldn't open \$filename for reading: \$!";
 
     my \%violated;
@@ -710,15 +693,6 @@ sub not_in_file_ok {
     }
 }
 
-not_in_file_ok(README =>
-    "The README is used..."       => qr/The README is used/,
-    "'version information here'"  => qr/to provide version information/,
-);
-
-not_in_file_ok(Changes =>
-    "placeholder date/time"       => qr(Date/time)
-);
-
 sub module_boilerplate_ok {
     my (\$module) = \@_;
     not_in_file_ok(\$module =>
@@ -728,10 +702,23 @@ sub module_boilerplate_ok {
     );
 }
 
-HERE
+TODO: {
+  local \$TODO = "Need to replace the boilerplate text";
 
-    $t_files{'boilerplate.t'}
-        .= "module_boilerplate_ok('$self->{module_file}{$_}');\n" for @modules;
+  not_in_file_ok(README =>
+    "The README is used..."       => qr/The README is used/,
+    "'version information here'"  => qr/to provide version information/,
+  );
+
+  not_in_file_ok(Changes =>
+    "placeholder date/time"       => qr(Date/time)
+  );
+
+$module_boilerplate_tests
+
+}
+
+HERE
 
     return %t_files;
 }
@@ -741,7 +728,7 @@ sub _create_t {
     my $filename = shift;
     my $content = shift;
 
-    my @dirparts = ( $self->{basedir}, "t" );
+    my @dirparts = ( $self->{basedir}, 't' );
     my $tdir = File::Spec->catdir( @dirparts );
     if ( not -d $tdir ) {
         local @ARGV = $tdir;
@@ -750,9 +737,7 @@ sub _create_t {
     }
 
     my $fname = File::Spec->catfile( @dirparts, $filename );
-    open( my $fh, ">", $fname ) or die "Can't create $fname: $!\n";
-    print $fh $content;
-    close $fh;
+    $self->create_file( $fname, $content );
     $self->progress( "Created $fname" );
 
     return "t/$filename";
@@ -770,14 +755,11 @@ sub create_MANIFEST {
     my $self = shift;
     my @files = @_;
 
-    my $fname = File::Spec->catfile( $self->{basedir}, "MANIFEST" );
-    open( my $fh, ">", $fname ) or die "Can't create $fname: $!\n";
-    print $fh $self->MANIFEST_guts(@files);
-    close $fh;
-
+    my $fname = File::Spec->catfile( $self->{basedir}, 'MANIFEST' );
+    $self->create_file( $fname, $self->MANIFEST_guts(@files) );
     $self->progress( "Created $fname" );
 
-    return "MANIFEST";
+    return 'MANIFEST';
 }
 
 =head2 MANIFEST_guts( @files )
@@ -791,9 +773,67 @@ sub MANIFEST_guts {
     my $self = shift;
     my @files = sort @_;
 
-    my $content = '';
-    $content .= "$_\n" for @files;
-    return $content;
+    return join( "\n", @files, '' );
+}
+
+=head2 create_build( )
+
+This method creates the build file(s) and puts together some build
+instructions.  The builders currently supported are:
+
+ExtUtils::MakeMaker
+Module::Build
+Module::Install
+
+=cut
+
+sub create_build {
+  my $self = shift;
+
+  # pass one: pull the builders out of $self->{builder}
+  my @tmp =
+    ref $self->{builder} eq 'ARRAY' ? @{$self->{builder}} : $self->{builder};
+
+  my @builders;
+  my $COMMA = q{,};
+  # pass two: expand comma-delimited builder lists
+  foreach my $builder (@tmp) {
+    push( @builders, split($COMMA, $builder) );
+  }
+
+  my $builder_set = Module::Starter::BuilderSet->new();
+
+  # Remove mutually exclusive and unsupported builders
+  @builders = $builder_set->check_compatibility( @builders );
+
+  # compile some build instructions, create a list of files generated
+  # by the builders' create_* methods, and call said methods
+
+  my @build_instructions;
+  my @files;
+
+  foreach my $builder ( @builders ) {
+    if ( !@build_instructions ) {
+      push( @build_instructions,
+            'To install this module, run the following commands:'
+          );
+    } else {
+      push( @build_instructions,
+            "Alternatively, to install with $builder, you can ".
+            "use the following commands:"
+          );
+    }
+    push( @files, $builder_set->file_for_builder($builder) );
+    my @commands = $builder_set->instructions_for_builder($builder);
+    push( @build_instructions, join("\n", map { "\t$_" } @commands) );
+
+    my $build_method = $builder_set->method_for_builder($builder);
+    $self->$build_method($self->{main_module})
+  }
+
+  return( files        => [ @files ],
+          instructions => join( "\n\n", @build_instructions ),
+        );
 }
 
 =head2 create_cvsignore( )
@@ -806,10 +846,8 @@ knows to ignore certain files.
 sub create_cvsignore {
     my $self = shift;
 
-    my $fname = File::Spec->catfile( $self->{basedir}, ".cvsignore" );
-    open( my $fh, ">", $fname ) or die "Can't create $fname: $!\n";
-    print $fh $self->cvsignore_guts();
-    close $fh;
+    my $fname = File::Spec->catfile( $self->{basedir}, '.cvsignore' );
+    $self->create_file( $fname, $self->cvsignore_guts() );
     $self->progress( "Created $fname" );
 
     return; # Not a file that goes in the MANIFEST
@@ -847,8 +885,26 @@ C<verbose> tells us whether we're in verbose mode.
 
 =cut
 
-sub verbose { $_[0]->{verbose} }
+sub verbose { return shift->{verbose} }
 
+=head2 create_file( $fname, @content_lines )
+
+Creates I<$fname>, dumps I<@content_lines> in it, and closes it.
+Dies on any error.
+
+=cut
+
+sub create_file {
+    my $self = shift;
+    my $fname = shift;
+
+    my @content = @_;
+    open( my $fh, '>', $fname ) or confess "Can't create $fname: $!\n";
+    print {$fh} @content;
+    close $fh or die "Can't close $fname: $!\n";
+
+    return;
+}
 
 =head2 progress( @list )
 
@@ -874,9 +930,11 @@ be notified of progress on your bug as I make changes.
 
 Andy Lester, C<< <andy@petdance.com> >>
 
+C.J. Adams-Collier, C<< <cjac@colliertech.org> >>
+
 =head1 Copyright & License
 
-Copyright 2005 Andy Lester, All Rights Reserved.
+Copyright 2005-2007 Andy Lester and C.J. Adams-Collier, All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -886,4 +944,174 @@ employers of the various contributors to the code.
 
 =cut
 
+sub _module_header {
+    my $self = shift;
+    my $module = shift;
+    my $rtname = shift;
+    my $content = <<"HERE";
+package $module;
+
+use warnings;
+use strict;
+
+\=head1 NAME
+
+$module - The great new $module!
+
+\=head1 VERSION
+
+Version 0.01
+
+\=cut
+
+our \$VERSION = '0.01';
+HERE
+    return $content;
+}
+
+sub _module_bugs {
+    my $self   = shift;
+    my $module = shift;
+    my $rtname = shift;
+
+    my $bug_email = "bug-$rtname at rt.cpan.org";
+    my $bug_link  =
+      "http://rt.cpan.org/NoAuth/ReportBug.html?Queue=$self->{distro}";
+
+    my $content = <<"HERE";
+\=head1 BUGS
+
+Please report any bugs or feature requests to C<$bug_email>, or through
+the web interface at L<$bug_link>.  I will be notified, and then you'll
+automatically be notified of progress on your bug as I make changes.
+
+HERE
+
+    return $content;
+}
+
+sub _module_support {
+    my $self   = shift;
+    my $module = shift;
+    my $rtname = shift;
+
+    my $content = qq[
+\=head1 SUPPORT
+
+You can find documentation for this module with the perldoc command.
+
+    perldoc $self->{main_module}
+];
+    my @reference_links = _reference_links();
+
+    return unless @reference_links;
+    $content .= qq[
+
+You can also look for information at:
+
+\=over 4
+];
+
+    foreach my $ref (@reference_links){
+      my $title;
+      my $link = sprintf($ref->{link}, $self->{distro});
+
+      $title = "$ref->{nickname}: " if exists $ref->{nickname};
+      $title .= $ref->{title};
+      $content .= qq[
+\=item * $title
+
+L<$link>
+];
+    }
+    $content .= qq[
+\=back
+];
+    return $content;
+}
+
+sub _module_license {
+    my $self = shift;
+
+    my $module = shift;
+    my $rtname = shift;
+
+    my $license_blurb = $self->_license_blurb();
+    my $year          = $self->_thisyear();
+
+    my $content = qq[
+\=head1 COPYRIGHT & LICENSE
+
+Copyright $year $self->{author}, all rights reserved.
+
+$license_blurb
+];
+
+    return $content;
+}
+
+sub module_guts {
+    my $self = shift;
+    my $module = shift;
+    my $rtname = shift;
+
+    # Sub-templates
+    my $header  = $self->_module_header($module, $rtname);
+    my $bugs    = $self->_module_bugs($module, $rtname);
+    my $support = $self->_module_support($module, $rtname);
+    my $license = $self->_module_license($module, $rtname);
+
+    my $content = <<"HERE";
+$header
+
+\=head1 SYNOPSIS
+
+Quick summary of what the module does.
+
+Perhaps a little code snippet.
+
+    use $module;
+
+    my \$foo = $module->new();
+    ...
+
+\=head1 EXPORT
+
+A list of functions that can be exported.  You can delete this section
+if you don't export anything, such as for a purely object-oriented module.
+
+\=head1 FUNCTIONS
+
+\=head2 function1
+
+\=cut
+
+sub function1 {
+}
+
+\=head2 function2
+
+\=cut
+
+sub function2 {
+}
+
+\=head1 AUTHOR
+
+$self->{author}, C<< <$self->{email_obfuscated}> >>
+
+$bugs
+
+$support
+
+\=head1 ACKNOWLEDGEMENTS
+
+$license
+
+\=cut
+
+1; # End of $module
+HERE
+    return $content;
+}
 1;
